package de.ugoe.cs.rwm.mocci;

import java.nio.file.Path;
import java.nio.file.Paths;
import org.eclipse.emf.ecore.resource.Resource;
import org.json.JSONArray;

import de.ugoe.cs.rwm.docci.MartDeployer;
import de.ugoe.cs.rwm.docci.connector.Connector;
import de.ugoe.cs.rwm.docci.connector.LocalhostConnector;
import de.ugoe.cs.rwm.docci.executor.MartExecutor;

/**Making javadoc happy.
 * @author erbel
 *
 */
public class MAPE {
	protected static final Path RUNTIMEPATH = Paths.get(System.getProperty("user.home") + "/.rwm/runtime.occic");
	static Connector conn = new LocalhostConnector("localhost", 8080, "ubuntu");
	static MartDeployer deployer = new MartDeployer(conn);
	static MartExecutor executor = new MartExecutor(conn);
	static Resource runtimeModel;
	static int interval = 10000;
	
	/**Making javadoc happy.
	 * @param args Making javadoc happy.
	 */
	public static void main(String[] args) {
		System.out.println("Starting MAPE loop");
		RegistryAndLoggerSetup.setup();
		initialDeploy(conn);
		
		while(true) {
			try {
				System.out.println("\n--------------------Waiting for new MAPE-K Cycle: Sleeping " + interval +"--------------------");
				Thread.sleep(interval);
				
				Monitor monitor = monitor();
				String analysis = analyze(monitor);
				runtimeModel = plan(analysis);
				execute(runtimeModel);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
		}
	}

private static void initialDeploy(Connector conn) {
		InitialDeployment depl = new InitialDeployment(conn);
		depl.deploy();
	}

public static Monitor monitor() {
		int critCPUs = getNumberOfCriticalCPUs();
		int noneCPUs = getNumberOfNoneCPUs();
		int allCPUs = getNumberOfAllCPUs();
		Monitor mon = new Monitor(critCPUs, noneCPUs, allCPUs);
		System.out.println("Monitor: " + "Monitored CPUs: " + allCPUs + "| Critical CPUs: "+ critCPUs + "| None CPUs: " + noneCPUs);
		return mon;
}
	

public static String analyze(Monitor monitor) {
	int noneCPUs = monitor.getNoneCPUs();
	int critCPUs = monitor.getCritCPUs();
	int allCPUs = monitor.getAllCPUs();
	
	if(noneCPUs == 0 && critCPUs > allCPUs/2 && allCPUs <= 6){
		System.out.println("Analyze: Critical State Detected");
		return "upScale";
	} else {
		System.out.println("Analyze: Non Critical State Detected");
		return "downScale";
	}		
}
	
public static Resource plan(String analysis) {
	switch (analysis) {
		case "upScale": 	System.out.println("Plan: upScale!");
							UpScaler upscaler = new UpScaler(conn, RUNTIMEPATH);
							return upscaler.upScaleNodes();
		case "downScale": 	System.out.println("Plan: downScale!");
							DownScaler downscaler = new DownScaler(conn, RUNTIMEPATH);
							return downscaler.downScaleNodes();
	}
	return null;
}


public static void execute(Resource runtimeModel) {
		System.out.println("Execute: Deploying adjusted Model");
		deployer.deploy(runtimeModel);
		
	}



private static int getNumberOfNoneCPUs() {
	String query = "/monitorableproperty?attribute=monitoring.result&value=None";
	String result = executor.executeGetOperation(query);
	if(result.equals("{ }") == false) {
		String substring = result.substring(result.indexOf("["), (result.lastIndexOf("]")+1));
		JSONArray arr = new JSONArray(substring);
		if(arr.length() == 0) {
			return 1;
		} else {
			return arr.length();
		}
	} else {
		return 0;
	}
	
}

private static int getNumberOfCriticalCPUs() {
	String query = "/monitorableproperty?attribute=monitoring.result&value=Critical";
	String result = executor.executeGetOperation(query);
	if(result.equals("{ }") == false) {
		String substring = result.substring(result.indexOf("["), (result.lastIndexOf("]")+1));
		JSONArray arr = new JSONArray(substring);
		if(arr.length() == 0) {
			return 1;
		} else {
			return arr.length();
		}
	} else {
		return 0;
	}	
}

private static int getNumberOfAllCPUs() {
	String query = "/monitorableproperty?attribute=monitoring.property&value=CPU";
	String result = executor.executeGetOperation(query);
	if(result.equals("{ }") == false) {
		String substring = result.substring(result.indexOf("["), (result.lastIndexOf("]")+1));
		JSONArray arr = new JSONArray(substring);
		if(arr.length() == 0) {
			return 1;
		} else {
			return arr.length();
		}
	} else {
		return 0;
	}	
}

}